@@ -38,6 +38,23 @@
 /* Define if you have vsyslog */
 #undef HAVE_VSYSLOG
 
+/* Define if you have atexit */
+#undef HAVE_ATEXIT
+
+/* Define if you have on_exit */
+#undef HAVE_ON_EXIT
+
+/* Define to help us deduce a 32-bit type (required for Kerberos) */
+#undef SIZEOF_INT
+#undef SIZEOF_SHORT
+#undef SIZEOF_LONG
+
+/* Define if you want POP2 support compiled in */
+#undef POP2_ENABLE
+
+/* Define if you want OPIE support compiled in */
+#undef OPIE_ENABLE
+
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
