@@ -4,14 +4,20 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <errno.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/fcntl.h>	/* Solaris 2.5 requires this */
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#ifdef HAVE_SYS_FCNTL_H
+#include <sys/fcntl.h>
+#endif /* HAVE_SYS_FCNTL_H */
+#include <sys/stat.h>	/* get umask(2) prototyped */
 
 #if defined(HAVE_UNISTD_H)
 #include <unistd.h>
@@ -25,7 +31,11 @@
 #include <unix.h>
 #endif
 
-#if defined(HAVE_SGTTY_H)
+#if defined(HAVE_TERMIOS_H)
+#  include <termios.h>		/* for TIOCNOTTY under Linux */
+#endif
+
+#if !defined(TIOCNOTTY) && defined(HAVE_SGTTY_H)
 #  include <sgtty.h>		/* for TIOCNOTTY under NEXTSTEP */
 #endif
 
@@ -50,7 +60,11 @@ sigchld_handler (int sig)
 #endif
 
 #if 	defined(HAVE_WAIT3)
+#ifdef oldhpux	/* HP-UX fixed this sometime between 9.01 and 10.20 */
+  while ((pid = wait3(&status, WNOHANG, (int *) 0)) > 0)
+#else
   while ((pid = wait3(&status, WNOHANG, (struct rusage *) 0)) > 0)
+#endif
     ; /* swallow 'em up. */
 #elif 	defined(HAVE_WAITPID)
   while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
