@@ -17,10 +17,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 /* Written by David MacKenzie <djm@gnu.ai.mit.edu>.
  * Heavily modified by Dave Bodenstab and ESR.
+ * Bludgeoned into submission for SunOS 4.1.3 by
+ *     Chris Cheyney <cheyney@netcom.com>.
+ * Now it works even when the return from vprintf is unreliable.
  */
 
 #ifdef HAVE_CONFIG_H
-# include <config.h>
+# include "config.h"
 #endif
 
 #include <stdio.h>
@@ -114,27 +117,6 @@ private_strerror (errnum)
    Exit with status STATUS if it is nonzero.  */
 /* VARARGS */
 
-#if !defined(HAVE_VSYSLOG) && !defined(VA_START)
-int vsyslog(priority, message, va_alist)
-int priority;
-char *message;
-va_dcl
-{
-    va_list args;
-  
-    char *string;
-  
-    string = (char *)malloc(LINELEN);
- 
-    va_start(args);
-    vsprintf(string, message, args);
-    va_end(args);
- 
-    syslog(priority, string);
-    free(string);
-}
-#endif
-
 void
 #if defined(VA_START) && __STDC__
 error (int status, int errnum, const char *message, ...)
@@ -154,7 +136,7 @@ error (status, errnum, message, va_alist)
   if (partial_message_size_used != 0)
     {
       partial_message_size_used = 0;
-      error (0, 0, "%s (message incomplete)", partial_message);
+      error (0, 0, "%s (log message incomplete)", partial_message);
     }
 
 #if defined(HAVE_SYSLOG)
@@ -162,12 +144,12 @@ error (status, errnum, message, va_alist)
     {
       int priority;
 
-# ifdef VA_START
+#ifdef VA_START
       VA_START (args, message);
-# endif
+#endif
       priority = status? LOG_ALERT : errnum? LOG_ERR : LOG_INFO;
 
-      if (errnum)
+      if (errnum > 0)
         {
 	  char *msg = alloca (strlen (message) + 5);
 
@@ -175,22 +157,44 @@ error (status, errnum, message, va_alist)
 	  strcat (msg, ": %m");
 
 	  errno = errnum;
-# ifdef VA_START
+#ifdef HAVE_VSYSLOG
 	  vsyslog (priority, msg, args);
-	  va_end (args);
-# else
+#else
+	  {
+	  char *a1 = va_arg(args, char *);
+	  char *a2 = va_arg(args, char *);
+	  char *a3 = va_arg(args, char *);
+	  char *a4 = va_arg(args, char *);
+	  char *a5 = va_arg(args, char *);
+	  char *a6 = va_arg(args, char *);
+	  char *a7 = va_arg(args, char *);
+	  char *a8 = va_arg(args, char *);
 	  syslog (priority, msg, a1, a2, a3, a4, a5, a6, a7, a8);
-# endif
+	  }
+#endif
 	}
       else
         {
-# ifdef VA_START
+#ifdef HAVE_VSYSLOG
 	  vsyslog (priority, message, args);
-	  va_end (args);
-# else
+#else
+	  {
+	  char *a1 = va_arg(args, char *);
+	  char *a2 = va_arg(args, char *);
+	  char *a3 = va_arg(args, char *);
+	  char *a4 = va_arg(args, char *);
+	  char *a5 = va_arg(args, char *);
+	  char *a6 = va_arg(args, char *);
+	  char *a7 = va_arg(args, char *);
+	  char *a8 = va_arg(args, char *);
 	  syslog (priority, message, a1, a2, a3, a4, a5, a6, a7, a8);
-# endif
+	  }
+#endif
 	}
+
+#ifdef VA_START
+      va_end(args);
+#endif
     }
   else
 #endif
@@ -220,8 +224,15 @@ error (status, errnum, message, va_alist)
       fprintf (stderr, message, a1, a2, a3, a4, a5, a6, a7, a8);
 #endif
 
-      if (errnum)
-	fprintf (stderr, ": %s", strerror (errnum));
+      if (errnum) {
+	char *tmps = strerror(errnum);
+	if (tmps) {
+	  fprintf (stderr, ": %s", tmps);
+	}
+	else {
+	  fprintf (stderr, ": Error %d", errnum);
+	}
+      }
       putc ('\n', stderr);
       fflush (stderr);
     }
@@ -247,8 +258,8 @@ int foreground;
    empty) by calling error_complete().
    If an intervening call to error() occurs when a partially constructed
    message exists, then, in an attempt to keep the messages in their proper
-   sequence, the partial message will be printed as-is (with a trailing newline)
-   before error() prints its message.
+   sequence, the partial message will be printed as-is (with a trailing 
+   newline) before error() prints its message. */
 /* VARARGS */
 
 void
@@ -269,13 +280,13 @@ error_build (message, va_alist)
   if (partial_message_size == 0)
     {
       partial_message_size_used = 0;
-      partial_message_size = 512;
+      partial_message_size = 2048;
       partial_message = xmalloc (partial_message_size);
     }
   else
-    if (partial_message_size - partial_message_size_used < 256)
+    if (partial_message_size - partial_message_size_used < 1024)
       {
-        partial_message_size += 512;
+        partial_message_size += 2048;
         partial_message = xrealloc (partial_message, partial_message_size);
       }
 
@@ -294,11 +305,12 @@ error_build (message, va_alist)
 	  break;
 	}
 
-      partial_message_size += 512;
+      partial_message_size += 2048;
       partial_message = xrealloc (partial_message, partial_message_size);
     }
 #else
-  partial_message_size_used += vsprintf (partial_message + partial_message_size_used, message, args);
+  vsprintf (partial_message + partial_message_size_used, message, args);
+  partial_message_size_used += strlen(partial_message+partial_message_size_used);
 
   /* Attempt to catch memory overwrites... */
   if (partial_message_size_used >= partial_message_size)
@@ -322,7 +334,7 @@ error_build (message, va_alist)
 	  break;
 	}
 
-      partial_message_size += 512;
+      partial_message_size += 2048;
       partial_message = xrealloc (partial_message, partial_message_size);
     }
 #else
@@ -347,7 +359,7 @@ error_build (message, va_alist)
 /* Complete an error message by appending MESSAGE, which is a printf-style
    format string with optional args, to the existing error message (which may
    be empty.)  The completed error message is then printed (and reset to
-   empty.)
+   empty.) */
 /* VARARGS */
 
 void
@@ -370,13 +382,13 @@ error_complete (status, errnum, message, va_alist)
   if (partial_message_size == 0)
     {
       partial_message_size_used = 0;
-      partial_message_size = 512;
+      partial_message_size = 2048;
       partial_message = xmalloc (partial_message_size);
     }
   else
-    if (partial_message_size - partial_message_size_used < 256)
+    if (partial_message_size - partial_message_size_used < 1024)
       {
-        partial_message_size += 512;
+        partial_message_size += 2048;
         partial_message = xrealloc (partial_message, partial_message_size);
       }
 
@@ -395,11 +407,12 @@ error_complete (status, errnum, message, va_alist)
 	  break;
 	}
 
-      partial_message_size += 512;
+      partial_message_size += 2048;
       partial_message = xrealloc (partial_message, partial_message_size);
     }
 #else
-  partial_message_size_used += vsprintf (partial_message + partial_message_size_used, message, args);
+  vsprintf (partial_message + partial_message_size_used, message, args);
+  partial_message_size_used += strlen(partial_message+partial_message_size_used);
 
   /* Attempt to catch memory overwrites... */
   if (partial_message_size_used >= partial_message_size)
@@ -423,7 +436,7 @@ error_complete (status, errnum, message, va_alist)
 	  break;
 	}
 
-      partial_message_size += 512;
+      partial_message_size += 2048;
       partial_message = xrealloc (partial_message, partial_message_size);
     }
 #else
