@@ -7,15 +7,16 @@
 #define		P_POP2		2
 #define		P_POP3		3
 #define		P_IMAP		4
-#define		P_APOP		5
-#define		P_RPOP		6
-#define		P_ETRN		7
+#define		P_IMAP_K4	5
+#define		P_APOP		6
+#define		P_RPOP		7
+#define		P_ETRN		8
 
 #define		KPOP_PORT	1109
 
-/* authentication types */
-#define		A_PASSWORD	0	/* passwords in cleartext */
-#define		A_KERBEROS	1	/* get Kerberos V4 ticket */
+/* preauthentication types */
+#define		A_PASSWORD	0	/* password or inline authentication */
+#define		A_KERBEROS_V4	1	/* preauthenticate w/ Kerberos V4 */
 
 /* definitions for buffer sizes -- somewhat arbitrary */
 #define		POPBUFSIZE	512	/* per RFC 937 */
@@ -35,10 +36,13 @@
 #define		PS_SYNTAX	5	/* command-line syntax error */
 #define		PS_IOERR	6	/* bad permissions on rc file */
 #define		PS_ERROR	7	/* protocol error */
-#define		PS_EXCLUDE	8	/* exclusion error */
-#define         PS_SMTP         9       /* SMTP error */
-#define		PS_UNDEFINED	10	/* something I hadn't thought of */
-#define		PS_TRANSIENT	11	/* transient failure (internal use) */
+#define		PS_EXCLUDE	8	/* client-side exclusion error */
+#define		PS_LOCKBUSY	9	/* server responded lock busy */
+#define		PS_SMTP         10      /* SMTP error */
+#define		PS_UNDEFINED	11	/* something I hadn't thought of */
+#define		PS_TRANSIENT	12	/* transient failure (internal use) */
+#define		PS_REFUSED	13	/* mail refused (internal use) */
+#define		PS_RETAINED	24	/* message retained (internal use) */
 
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
@@ -62,19 +66,32 @@ struct idlist
     struct idlist *next;
 };
 
+/*
+ * We #ifdef this and use flag rather than bool
+ * to avoid a type clash with curses.h
+ */
+#ifndef TRUE
+#define FALSE	0
+#define TRUE	1
+#endif /* TRUE */
+typedef	char	flag;
 
 struct hostdata		/* shared among all user connections to given server */
 {
     /* rc file data */
-    struct idlist *names;		/* server name first, then akas */
+    char *pollname;			/* poll label of host */
+    char *via;				/* "true" server name if non-NULL */
+    struct idlist *akalist;		/* server name first, then akas */
     struct idlist *localdomains;	/* list of pass-through domains */
-    int protocol;
-    int port;
-    int authenticate;
-    int timeout;
-    char *envelope;
-    int skip;
-    int dns;
+    int protocol;			/* protocol type */
+    int port;				/* TCP/IP service port number */
+    int interval;			/* # cycles to skip between polls */
+    int preauthenticate;		/* preauthentication mode to try */
+    int timeout;			/* inactivity timout in seconds */
+    char *envelope;			/* envelope address list header */
+    flag skip;				/* suppress poll in implicit mode? */
+    flag dns;				/* do DNS lookup on multidrop? */
+    flag uidl;				/* use RFC1725 UIDLs? */
 
 #ifdef linux
     char *interface;
@@ -84,9 +101,9 @@ struct hostdata		/* shared among all user connections to given server */
 #endif /* linux */
 
     /* computed for internal use */
-#ifdef HAVE_GETHOSTBYNAME
-    char *canonical_name;		/* DNS canonical name of server host */
-#endif /* HAVE_GETHOSTBYNAME */
+    int poll_count;			/* count of polls so far */
+    char *queryname;			/* name to attempt DNS lookup on */
+    char *truename;			/* "true name" of server host */
     struct hostdata *lead_server;	/* ptr to lead query for this server */
     int esmtp_options;
 };
@@ -97,36 +114,40 @@ struct query
     struct hostdata server;
 
     /* per-user data */
-    struct idlist *localnames;		/* including calling user's name */
+    struct idlist *localnames;	/* including calling user's name */
     int wildcard;		/* should unmatched names be passed through */
-    char *remotename;
-    char *password;
-    char *mailbox;
-    struct idlist *smtphunt;
-    char *smtphost;
-    char *mda;
-    char *preconnect;
+    char *remotename;		/* remote login name to use */
+    char *password;		/* remote password to use */
+    struct idlist *mailboxes;	/* list of mailboxes to check */
+    struct idlist *smtphunt;	/* list of SMTP hosts to try forwarding to */
+    char *smtphost;		/* actual SMTP host to point to */
+    char *mda;			/* local MDA to pass mail to */
+    char *preconnect;		/* pre-connection command to execute */
+    char *postconnect;		/* post-connection command to execute */
 
     /* per-user control flags */
-    int keep;
-    int fetchall;
-    int flush;
-    int rewrite;
-    int stripcr;
-    int limit;
-    int fetchlimit;
-    int batchlimit;
+    flag keep;			/* if TRUE, leave messages undeleted */
+    flag fetchall;		/* if TRUE, fetch all (not just unseen) */
+    flag flush;			/* if TRUE, delete messages already seen */
+    flag rewrite;		/* if TRUE, canonicalize recipient addresses */
+    flag stripcr;		/* if TRUE, strip CRs in text */
+    flag forcecr;		/* if TRUE, force CRs before LFs in text */
+    flag pass8bits;		/* if TRUE, ignore Content-Transfer-Encoding */
+    flag dropstatus;		/* if TRUE, drop Status lines in mail */
+    int	limit;			/* limit size of retrieved messages */
+    int	fetchlimit;		/* max # msgs to get in single poll */
+    int	batchlimit;		/* max # msgs to pass in single SMTP session */
 
     /* unseen, previous state of mailbox (initially from .fetchids) */
     struct idlist *oldsaved, *newsaved;
 
     /* internal use */
-    int active;
+    flag active;		/* should we actually poll this server? */
     int errcount;		/* count transient errors in last pass */
-    struct query *next;		/* next query control block in chain */
-    FILE *smtp_sockfp;		/* socket descriptor for SMTP connection */
+    int smtp_socket;		/* socket descriptor for SMTP connection */
     unsigned int uid;		/* UID of user to deliver to */
     char digest [DIGESTLEN];	/* md5 digest buffer */
+    struct query *next;		/* next query control block in chain */
 };
 
 #define MULTIDROP(ctl)	(ctl->wildcard || \
@@ -136,16 +157,19 @@ struct method
 {
     char *name;			/* protocol name */
     int	port;			/* service port */
-    int tagged;			/* if true, generate & expect command tags */
-    int delimited;		/* if true, accept "." message delimiter */
+    flag tagged;		/* if true, generate & expect command tags */
+    flag delimited;		/* if true, accept "." message delimiter */
+    flag force_getsizes;	/* if true, fetch's size return unreliable */
     int (*parse_response)();	/* response_parsing function */
     int (*getauth)();		/* authorization fetcher */
     int (*getrange)();		/* get message range to fetch */
     int (*getsizes)();		/* get sizes of messages */
     int (*is_old)();		/* check for old message */
-    int (*fetch)();		/* fetch a given message */
+    int (*fetch_headers)();	/* fetch FROM headera given message */
+    int (*fetch_body)();	/* fetch a given message */
     int (*trail)();		/* eat trailer of a message */
     int (*delete)();		/* delete method */
+    flag (*retain_hdr)();	/* if this returns TRUE, retain message */
     char *exit_cmd;		/* exit command */
 };
 
@@ -155,28 +179,33 @@ extern char tag[TAGLEN];
 /* list of hosts assembled from run control file and command line */
 extern struct query cmd_opts, *querylist;
 
+/* what's returned by envquery */
+extern void envquery(int, char **);
+char *user, *home, *fetchmailhost;
+
 /* controls the detail level of status/progress messages written to stderr */
 extern int outlevel;    	/* see the O_.* constants above */
 extern int yydebug;		/* enable parse debugging */
 
 /* daemon mode control */
 extern int poll_interval;	/* poll interval in seconds */
-extern int nodetach;		/* if TRUE, don't detach daemon process */
+extern flag nodetach;		/* if TRUE, don't detach daemon process */
 extern char *logfile;		/* log file for daemon mode */
-extern int use_syslog;		/* if --syslog was set */
-extern int quitmode;		/* if --quit was set */
-extern int check_only;		/* if --check was set */
+extern flag use_syslog;		/* if --syslog was set */
+extern flag quitmode;		/* if --quit was set */
+extern flag check_only;		/* if --check was set */
 extern char *cmd_logfile;	/* if --logfile was set */
+extern int cmd_daemon;		/* if --daemon was set */
 
 /* these get computed */
 extern int batchcount;		/* count of messages sent in current batch */
-extern int peek_capable;	/* can we read msgs without setting seen? */
+extern flag peek_capable;	/* can we read msgs without setting seen? */
 
 /* miscellaneous global controls */
 extern char *rcfile;		/* path name of rc file */
 extern char *idfile;		/* path name of UID file */
 extern int linelimit;		/* limit # lines retrieved per site */
-extern int versioninfo;		/* emit only version info */
+extern flag versioninfo;	/* emit only version info */
 extern char *user;		/* name of invoking user */
 extern char *fetchmailhost;	/* the name of the host running fetchmail */
 
@@ -186,21 +215,19 @@ void error_init(int foreground);
 void error (int status, int errnum, const char *format, ...);
 void error_build (const char *format, ...);
 void error_complete (int status, int errnum, const char *format, ...);
-void gen_send (FILE *sockfp, char *, ... );
-int gen_recv(FILE *sockfp, char *buf, int size);
-int gen_transact (FILE *sockfp, char *, ... );
+void error_at_line (int, int, const char *, unsigned int, const char *, ...);
+void gen_send (int sock, char *, ... );
+int gen_recv(int sock, char *buf, int size);
+int gen_transact (int sock, char *, ... );
 #else
 void error ();
 void error_build ();
 void error_complete ();
+void error_at_line ();
 void gen_send ();
 int gen_transact ();
 #endif
 
-void *xmalloc(int);
-void *xrealloc(void *, int);
-char *xstrdup(const char *);
-
 int do_protocol(struct query *, const struct method *);
 int doPOP2 (struct query *); 
 int doPOP3 (struct query *);
@@ -210,6 +237,7 @@ int doETRN (struct query *);
 void reply_hack(char *, const char *);
 char *nxtaddr(const char *);
 
+/* UID support */
 void initialize_saved_lists(struct query *, const char *);
 struct idlist *save_str(struct idlist **, int, const char *);
 void free_str_list(struct idlist **);
@@ -217,6 +245,9 @@ void save_str_pair(struct idlist **, const char *, const char *);
 void free_str_pair_list(struct idlist **);
 int delete_str(struct idlist **, int);
 int str_in_list(struct idlist **, const char *);
+int str_nr_in_list(struct idlist **, const char *);
+int count_list( struct idlist **idl );
+char *str_from_nr_list( struct idlist **idl, int number );
 char *str_find(struct idlist **, int);
 char *idpair_find(struct idlist **, const char *);
 void append_str_list(struct idlist **, struct idlist **);
@@ -229,7 +260,7 @@ void optmerge(struct query *, struct query *);
 char *MD5Digest (char *);
 int daemonize(const char *, void (*)(int));
 
-int prc_parse_file(const char *);
+int prc_parse_file(const char *, flag);
 int prc_filecheck(const char *);
 
 void interface_parse(char *, struct hostdata *);
@@ -239,11 +270,25 @@ int interface_approve(struct hostdata *);
 char *getpassword(char *);
 
 void escapes(const char *, char *);
+char *visbuf(const char *);
+char *showproto(int);
 
 void yyerror(const char *);
 int yylex(void);
 
-#define FALSE	0
-#define TRUE	1
+void to64frombits(unsigned char *, const unsigned char *, int);
+int from64tobits(char *, const char *);
+
+#if defined(HAVE_VOIDPOINTER)
+#define XMALLOCTYPE void
+#else
+#define XMALLOCTYPE char
+#endif
+
+XMALLOCTYPE *xmalloc(int);
+XMALLOCTYPE *xrealloc(XMALLOCTYPE *, int);
+char *xstrdup(const char *);
 
 #define STRING_DISABLED	(char *)-1
+
+/* fetchmail.h ends here */
