@@ -13,6 +13,7 @@
 
 #ifdef linux
 
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #if defined(STDC_HEADERS)
@@ -24,6 +25,7 @@
 #include <sys/ioctl.h>
 #include <netinet/in.h>
 #include <net/if.h>
+#include "config.h"
 #include "fetchmail.h"
 
 typedef struct {
@@ -43,10 +45,24 @@ static int _get_ifinfo_(int socket_fd, FILE *stats_file, const char *ifname,
 	int namelen = strlen(ifname);
 	struct ifreq request;
 	char *cp, buffer[256];
+	int found = 0;
 
 	/* initialize result */
 	memset((char *) ifinfo, 0, sizeof(ifinfo_t));
 
+	/* get the packet I/O counts */
+	while (fgets(buffer, sizeof(buffer) - 1, stats_file)) {
+		for (cp = buffer; *cp && *cp == ' '; ++cp);
+		if (!strncmp(cp, ifname, namelen) &&
+				cp[namelen] == ':') {
+			cp += namelen + 1;
+			sscanf(cp, "%d %*d %*d %*d %*d %d %*d %*d %*d %*d %*d",
+				&ifinfo->rx_packets, &ifinfo->tx_packets);
+                        found = 1;
+		}
+	}
+        if (!found) return (FALSE);
+
 	/* see if the interface is up */
 	strcpy(request.ifr_name, ifname);
 	if (ioctl(socket_fd, SIOCGIFFLAGS, &request) < 0)
@@ -68,21 +84,12 @@ static int _get_ifinfo_(int socket_fd, FILE *stats_file, const char *ifname,
 
 	/* get the netmask */
 	strcpy(request.ifr_name, ifname);
-	if (ioctl(socket_fd, SIOCGIFNETMASK, &request) >= 0)
-		ifinfo->netmask = ((struct sockaddr_in *)
-					(&request.ifr_netmask))->sin_addr;
+	if (ioctl(socket_fd, SIOCGIFNETMASK, &request) >= 0) {
+          ifinfo->netmask = ((struct sockaddr_in *)
+                             (&request.ifr_netmask))->sin_addr;
+          return (TRUE);
+        }
 
-	/* get the packet I/O counts */
-	while (fgets(buffer, sizeof(buffer) - 1, stats_file)) {
-		for (cp = buffer; *cp && *cp == ' '; ++cp);
-		if (!strncmp(cp, ifname, namelen) &&
-				cp[namelen] == ':') {
-			cp += namelen + 1;
-			sscanf(cp, "%d %*d %*d %*d %*d %d %*d %*d %*d %*d %*d",
-				&ifinfo->rx_packets, &ifinfo->tx_packets);
-			return(TRUE);
-		}
-	}
 	return(FALSE);
 }
 
@@ -93,7 +100,7 @@ static int get_ifinfo(const char *ifname, ifinfo_t *ifinfo)
 	int result;
 
 	if (socket_fd < 0 || !stats_file)
-		result = -1;
+		result = FALSE;
 	else
 		result = _get_ifinfo_(socket_fd, stats_file, ifname, ifinfo);
 	if (socket_fd >= 0)
@@ -121,7 +128,7 @@ void interface_parse(char *buf, struct hostdata *hp)
 		*cp2++ = '\000';
 
 	/* convert IP address and netmask */
-	hp->interface_pair = xmalloc(sizeof(struct interface_pair_s));
+	hp->interface_pair = (struct interface_pair_s *)xmalloc(sizeof(struct interface_pair_s));
 	if (!inet_aton(cp1, &hp->interface_pair->interface_address))
 		(void) error(PS_SYNTAX, 0, "invalid IP interface address");
 	if (!inet_aton(cp2, &hp->interface_pair->interface_mask))
@@ -136,6 +143,7 @@ void interface_note_activity(struct hostdata *hp)
 /* save interface I/O counts */
 {
 	ifinfo_t ifinfo;
+	struct query *ctl;
 
 	/* if not monitoring link, all done */
 	if (!hp->monitor)
@@ -143,11 +151,22 @@ void interface_note_activity(struct hostdata *hp)
 
 	/* get the current I/O stats for the monitored link */
 	if (get_ifinfo(hp->monitor, &ifinfo))
-		hp->monitor_io = ifinfo.rx_packets + ifinfo.tx_packets;
+		/* update this and preceeding host entries using the link
+		   (they were already set during this pass but the I/O
+		   count has now changed and they need to be re-updated)
+		*/
+		for (ctl = querylist; ctl; ctl = ctl->next) {
+			if (!strcmp(hp->monitor, ctl->server.monitor))
+				ctl->server.monitor_io =
+					ifinfo.rx_packets + ifinfo.tx_packets;
+			/* do NOT update host entries following this one */
+			if (&ctl->server == hp)
+				break;
+		}
 
 #ifdef	ACTIVITY_DEBUG
 	(void) error(0, 0, "activity on %s -noted- as %d", 
-		hp->names->id, hp->monitor_io);
+		hp->monitor, hp->monitor_io);
 #endif
 }
 
@@ -161,7 +180,7 @@ int interface_approve(struct hostdata *hp)
 		/* get interface info */
 		if (!get_ifinfo(hp->interface, &ifinfo)) {
 			(void) error(0, 0, "skipping poll of %s, %s down",
-				hp->names->id, hp->interface);
+				hp->pollname, hp->interface);
 			return(FALSE);
 		}
 		/* check the IP address (range) */
@@ -170,7 +189,7 @@ int interface_approve(struct hostdata *hp)
 				hp->interface_pair->interface_address.s_addr) {
 			(void) error(0, 0,
 				"skipping poll of %s, %s IP address excluded",
-				hp->names->id, hp->interface);
+				hp->pollname, hp->interface);
 			return(FALSE);
 		}
 	}
@@ -181,14 +200,14 @@ int interface_approve(struct hostdata *hp)
 
 #ifdef	ACTIVITY_DEBUG
 	(void) error(0, 0, "activity on %s checked as %d", 
-		hp->names->id, hp->monitor_io);
+		hp->monitor, hp->monitor_io);
 #endif
 	/* if monitoring, check link for activity if it is up */
 	if (get_ifinfo(hp->monitor, &ifinfo) &&
 			hp->monitor_io == ifinfo.rx_packets +
 				ifinfo.tx_packets) {
 		(void) error(0, 0, "skipping poll of %s, %s inactive",
-			hp->names->id, hp->monitor);
+			hp->pollname, hp->monitor);
 		return(FALSE);
 	}
 
