@@ -4,7 +4,7 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <pwd.h>
@@ -29,28 +29,33 @@
 #define LA_RCFILE	11
 #define LA_IDFILE	12
 #define LA_PROTOCOL	13
-#define LA_PORT		14
-#define LA_AUTHENTICATE	15
-#define LA_TIMEOUT	16
-#define LA_ENVELOPE	17
-#define LA_USERNAME	18
-#define LA_ALL          19
-#define LA_KILL		20
-#define	LA_KEEP		21
-#define LA_FLUSH        22
-#define LA_NOREWRITE	23
-#define LA_LIMIT	24
-#define LA_REMOTEFILE	25
-#define LA_SMTPHOST	26
-#define LA_BATCHLIMIT	27
-#define LA_FETCHLIMIT	28
-#define LA_MDA		29
-#define LA_INTERFACE    30
-#define LA_MONITOR      31
-#define LA_YYDEBUG	32
+#define LA_UIDL		14
+#define LA_PORT		15
+#define LA_AUTHENTICATE	16
+#define LA_TIMEOUT	17
+#define LA_ENVELOPE	18
+#define LA_USERNAME	19
+#define LA_ALL          20
+#define LA_NOKEEP	21
+#define	LA_KEEP		22
+#define LA_FLUSH        23
+#define LA_NOREWRITE	24
+#define LA_LIMIT	25
+#define LA_FOLDER	26
+#define LA_SMTPHOST	27
+#define LA_BATCHLIMIT	28
+#define LA_FETCHLIMIT	29
+#define LA_MDA		30
+#define LA_INTERFACE    31
+#define LA_MONITOR      32
+#define LA_YYDEBUG	33
 
-static char *shortoptions = "?Vcsvd:NqL:f:i:p:P:A:t:E:u:akKFnl:r:S:b:B:m:I:M:y";
-static struct option longoptions[] = {
+/* options still left: CDegGhHjJoOqQRTUwWxXYzZ */
+static const char *shortoptions = 
+	"?Vcsvd:NqL:f:i:p:UP:A:t:E:u:akKFnl:r:S:b:B:m:I:M:y";
+
+static const struct option longoptions[] = {
+/* this can be const because all flag fields are 0 and will never get set */
   {"help",	no_argument,	   (int *) 0, LA_HELP        },
   {"version",   no_argument,       (int *) 0, LA_VERSION     },
   {"check",	no_argument,	   (int *) 0, LA_CHECK       },
@@ -66,6 +71,7 @@ static struct option longoptions[] = {
 
   {"protocol",	required_argument, (int *) 0, LA_PROTOCOL    },
   {"proto",	required_argument, (int *) 0, LA_PROTOCOL    },
+  {"uidl",	no_argument,	   (int *) 0, LA_UIDL	     },
   {"port",	required_argument, (int *) 0, LA_PORT        },
   {"auth",	required_argument, (int *) 0, LA_AUTHENTICATE},
   {"timeout",	required_argument, (int *) 0, LA_TIMEOUT     },
@@ -75,13 +81,13 @@ static struct option longoptions[] = {
   {"username",  required_argument, (int *) 0, LA_USERNAME    },
 
   {"all",	no_argument,       (int *) 0, LA_ALL         },
-  {"kill",	no_argument,	   (int *) 0, LA_KILL        },
+  {"nokeep",	no_argument,	   (int *) 0, LA_NOKEEP        },
   {"keep",      no_argument,       (int *) 0, LA_KEEP        },
   {"flush",	no_argument,	   (int *) 0, LA_FLUSH       },
   {"norewrite",	no_argument,	   (int *) 0, LA_NOREWRITE   },
   {"limit",	required_argument, (int *) 0, LA_LIMIT       },
 
-  {"remote",    required_argument, (int *) 0, LA_REMOTEFILE  },
+  {"folder",    required_argument, (int *) 0, LA_FOLDER	     },
   {"smtphost",	required_argument, (int *) 0, LA_SMTPHOST    },
   {"batchlimit",required_argument, (int *) 0, LA_BATCHLIMIT  },
   {"fetchlimit",required_argument, (int *) 0, LA_FETCHLIMIT  },
@@ -115,8 +121,12 @@ struct query *ctl;	/* option record to be initialized */
     int ocount = 0;     /* count of destinations specified */
     int errflag = 0;   /* TRUE when a syntax error is detected */
     int option_index;
+    char buf[BUFSIZ], *cp;
+
+    cmd_daemon = -1;
 
     memset(ctl, '\0', sizeof(struct query));    /* start clean */
+    ctl->smtp_socket = -1;
 
     while (!errflag && 
 	   (c = getopt_long(argc,argv,shortoptions,
@@ -141,7 +151,7 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'd':
 	case LA_DAEMON:
-	    poll_interval = atoi(optarg);
+	    cmd_daemon = atoi(optarg);
 	    break;
 	case 'N':
 	case LA_NODETACH:
@@ -172,8 +182,6 @@ struct query *ctl;	/* option record to be initialized */
 		ctl->server.protocol = P_POP2;
 	    else if (strcasecmp(optarg,"pop3") == 0)
 		ctl->server.protocol = P_POP3;
-	    else if (strcasecmp(optarg,"imap") == 0)
-		ctl->server.protocol = P_IMAP;
 	    else if (strcasecmp(optarg,"apop") == 0)
 		ctl->server.protocol = P_APOP;
 	    else if (strcasecmp(optarg,"rpop") == 0)
@@ -182,8 +190,14 @@ struct query *ctl;	/* option record to be initialized */
 	    {
 		ctl->server.protocol = P_POP3;
 		ctl->server.port = KPOP_PORT;
-		ctl->server.authenticate =  A_KERBEROS;
+		ctl->server.preauthenticate =  A_KERBEROS_V4;
 	    }
+	    else if (strcasecmp(optarg,"imap") == 0)
+		ctl->server.protocol = P_IMAP;
+#ifdef KERBEROS_V4
+	    else if (strcasecmp(optarg,"imap-k4") == 0)
+		ctl->server.protocol = P_IMAP_K4;
+#endif /* KERBEROS_V4 */
 	    else if (strcasecmp(optarg,"etrn") == 0)
 		ctl->server.protocol = P_ETRN;
 	    else {
@@ -191,6 +205,10 @@ struct query *ctl;	/* option record to be initialized */
 		errflag++;
 	    }
 	    break;
+	case 'U':
+	case LA_UIDL:
+	    ctl->server.uidl = FLAG_TRUE;
+	    break;
 	case 'P':
 	case LA_PORT:
 	    ctl->server.port = atoi(optarg);
@@ -198,17 +216,21 @@ struct query *ctl;	/* option record to be initialized */
 	case 'A':
 	case LA_AUTHENTICATE:
 	    if (strcmp(optarg, "password") == 0)
-		ctl->server.authenticate = A_PASSWORD;
+		ctl->server.preauthenticate = A_PASSWORD;
 	    else if (strcmp(optarg, "kerberos") == 0)
-		ctl->server.authenticate = A_KERBEROS;
+		ctl->server.preauthenticate = A_KERBEROS_V4;
+	    else if (strcmp(optarg, "kerberos_v4") == 0)
+		ctl->server.preauthenticate = A_KERBEROS_V4;
 	    else {
-		fprintf(stderr,"Invalid authentication `%s' specified.\n", optarg);
+		fprintf(stderr,"Invalid preauthentication `%s' specified.\n", optarg);
 		errflag++;
 	    }
 	    break;
 	case 't':
 	case LA_TIMEOUT:
 	    ctl->server.timeout = atoi(optarg);
+	    if (ctl->server.timeout == 0)
+		ctl->server.timeout = -1;
 	    break;
 	case 'E':
 	case LA_ENVELOPE:
@@ -224,7 +246,7 @@ struct query *ctl;	/* option record to be initialized */
 	    ctl->fetchall = FLAG_TRUE;
 	    break;
 	case 'K':
-	case LA_KILL:
+	case LA_NOKEEP:
 	    ctl->keep = FLAG_FALSE;
 	    break;
 	case 'k':
@@ -237,28 +259,44 @@ struct query *ctl;	/* option record to be initialized */
 	    break;
 	case 'n':
 	case LA_NOREWRITE:
-	    ctl->rewrite = FLAG_TRUE;
+	    ctl->rewrite = FLAG_FALSE;
 	    break;
 	case 'l':
 	case LA_LIMIT:
 	    ctl->limit = atoi(optarg);
+	    if (ctl->limit == 0)
+		ctl->limit = -1;
 	    break;
 	case 'r':
-	case LA_REMOTEFILE:
-	    ctl->mailbox = xstrdup(optarg);
+	case LA_FOLDER:
+	    strcpy(buf, optarg);
+	    cp = strtok(buf, ",");
+	    do {
+		save_str(&ctl->mailboxes, -1, optarg);
+	    } while
+		((cp = strtok((char *)NULL, ",")));
 	    break;
 	case 'S':
 	case LA_SMTPHOST:
-	    save_str(&ctl->smtphunt, -1, optarg);
+	    strcpy(buf, optarg);
+	    cp = strtok(buf, ",");
+	    do {
+		save_str(&ctl->smtphunt, -1, optarg);
+	    } while
+		((cp = strtok((char *)NULL, ",")));
 	    ocount++;
 	    break;
 	case 'b':
 	case LA_BATCHLIMIT:
 	    ctl->batchlimit = atoi(optarg);
+	    if (ctl->batchlimit == 0)
+		ctl->batchlimit = -1;
 	    break;
 	case 'B':
 	case LA_FETCHLIMIT:
 	    ctl->fetchlimit = atoi(optarg);
+	    if (ctl->fetchlimit == 0)
+		ctl->fetchlimit = -1;
 	    break;
 	case 'm':
 	case LA_MDA:
@@ -269,7 +307,7 @@ struct query *ctl;	/* option record to be initialized */
 #ifdef	linux
 	case 'I':
 	case LA_INTERFACE:
-	    ctl->server.interface = xstrdup(optarg);
+	    interface_parse(optarg, &ctl->server);
 	    break;
 	case 'M':
 	case LA_MONITOR:
@@ -293,18 +331,6 @@ struct query *ctl;	/* option record to be initialized */
 	}
     }
 
-    if (check_only && poll_interval)
-    {
-	fputs("The --check and --daemon options aren't compatible.\n", stderr);
-	return(-1);
-    }
-
-    if (poll_interval == 0 && use_syslog)
-    {
-	fputs("The --syslog option is only valid with the --daemon option.\n", stderr);
-	return(-1);
-    }
-
     if (errflag || ocount > 1) {
 	/* squawk if syntax errors were detected */
 	fputs("usage:  fetchmail [options] [server ...]\n", stderr);
@@ -327,7 +353,12 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -M, --monitor     monitor interface for activity\n",stderr);
 #endif
 
+#ifdef KERBEROS_V4
+	fputs("  -p, --protocol    specify pop2, pop3, imap, apop, rpop, kpop, etrn, imap-k4\n", stderr);
+#else
 	fputs("  -p, --protocol    specify pop2, pop3, imap, apop, rpop, kpop, etrn\n", stderr);
+#endif /* KERBEROS_V4 */
+	fputs("  -U, --uidl        force the use of UIDLs (pop3 only)\n", stderr);
 	fputs("  -P, --port        TCP/IP service port to connect to\n",stderr);
 	fputs("  -A, --auth        authentication type (password or kerberos)\n",stderr);
 	fputs("  -t, --timeout     server nonresponse timeout\n",stderr);
@@ -335,7 +366,7 @@ struct query *ctl;	/* option record to be initialized */
 
 	fputs("  -u, --username    specify users's login on server\n", stderr);
 	fputs("  -a, --all         retrieve old and new messages\n", stderr);
-	fputs("  -K, --kill        delete new messages after retrieval\n", stderr);
+	fputs("  -K, --nokeep      delete new messages after retrieval\n", stderr);
 	fputs("  -k, --keep        save new messages after retrieval\n", stderr);
 	fputs("  -F, --flush       delete old messages from server\n", stderr);
 	fputs("  -n, --norewrite   don't rewrite header addresses\n", stderr);
@@ -344,7 +375,7 @@ struct query *ctl;	/* option record to be initialized */
 	fputs("  -S, --smtphost    set SMTP forwarding host\n", stderr);
 	fputs("  -b, --batchlimit  set batch limit for SMTP connections\n", stderr);
 	fputs("  -B, --fetchlimit  set fetch limit for server connections\n", stderr);
-	fputs("  -r, --remote      specify remote folder name\n", stderr);
+	fputs("  -r, --folder      specify remote folder name\n", stderr);
 	return(-1);
     }
 
