@@ -1,8 +1,7 @@
 /*
  * rfc822.c -- code for slicing and dicing RFC822 mail headers
  *
- * Copyright 1996 by Eric S. Raymond
- * All rights reserved.
+ * Copyright 1997 by Eric S. Raymond
  * For license terms, see the file COPYING in this directory.
  */
 
@@ -27,18 +26,19 @@ char *buf;		/* header to be hacked */
 const char *host;	/* server hostname */
 {
     char *from, *cp;
-    int parendepth, state, has_host_part;
+    int parendepth, state, has_bare_name_part, has_host_part;
 
-    if (strncmp("From: ", buf, 6)
-	&& strncmp("To: ", buf, 4)
-	&& strncmp("Reply-", buf, 6)
-	&& strncmp("Cc: ", buf, 4)
-	&& strncmp("Bcc: ", buf, 5)) {
+    if (strncasecmp("From: ", buf, 6)
+	&& strncasecmp("To: ", buf, 4)
+	&& strncasecmp("Reply-To: ", buf, 10)
+	&& strncasecmp("Return-Path: ", buf, 13)
+	&& strncasecmp("Cc: ", buf, 4)
+	&& strncasecmp("Bcc: ", buf, 5)) {
 	return;
     }
 
     parendepth = state = 0;
-    has_host_part = FALSE;
+    has_host_part = has_bare_name_part = FALSE;
     for (from = buf; *from; from++)
     {
 #ifdef TESTMAIN
@@ -69,7 +69,12 @@ const char *host;	/* server hostname */
 		    has_host_part = TRUE;
 		else if (*from == '"')
 		    state = 2;
-		else if ((*from == ',' || HEADER_END(from)) && !has_host_part)
+		/*
+		 * Not expanding on from[-1] == ';' deals with groupnames,
+		 * an obscure misfeature described in sections
+		 * 6.1, 6.2.6, and A.1.5 of the RFC822 standard.
+		 */
+		else if ((*from == ',' || HEADER_END(from)) && has_bare_name_part && !has_host_part && from[-1] != ';')
 		{
 		    int hostlen;
 
@@ -83,7 +88,9 @@ const char *host;	/* server hostname */
 		    memcpy(from, host, hostlen);
 		    from += strlen(from);
 		    has_host_part = TRUE;
-		}
+		} 
+		else if (!isspace(*from))
+		    has_bare_name_part = TRUE;
 		break;
 
 	    case 2:	/* we're in a string */
@@ -121,7 +128,7 @@ const char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 #ifdef TESTMAIN
     static const char *orighdr;
 #endif /* TESTMAIN */
-    int parendepth;
+    int parendepth = 0;
 
 #define START_HDR	0	/* before header colon */
 #define SKIP_JUNK	1	/* skip whitespace, \n, and junk */
@@ -186,6 +193,7 @@ const char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 	    else if (*hp == '(')	/* address comment -- ignore */
 	    {
 		parendepth = 1;
+		oldstate = SKIP_JUNK;
 		state = INSIDE_PARENS;    
 	    }
 	    else if (*hp == '<')	/* begin <address> */
@@ -213,6 +221,7 @@ const char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 	    else if (*hp == '(')  	/* beginning of comment */
 	    {
 		parendepth = 1;
+		oldstate = BARE_ADDRESS;
 		state = INSIDE_PARENS;    
 	    }
 	    else if (*hp == '<')  	/* beginning of real address */
@@ -240,7 +249,7 @@ const char *hdr;	/* header to be parsed, NUL to continue previous hdr */
 	    else if (*hp == ')')
 		--parendepth;
 	    if (parendepth == 0)
-		state = SKIP_JUNK;
+		state = oldstate;
 	    break;
 
 	case INSIDE_BRACKETS:	/* possible <>-enclosed address */
@@ -310,11 +319,11 @@ main(int argc, char *argv[])
     {
 	if (buf[0] == ' ' || buf[0] == '\t')
 	    strcat(longbuf, buf);
-	else if (!strncmp("From: ", buf, 6)
-		    || !strncmp("To: ", buf, 4)
-		    || !strncmp("Reply-", buf, 6)
-		    || !strncmp("Cc: ", buf, 4)
-		    || !strncmp("Bcc: ", buf, 5))
+	else if (!strncasecmp("From: ", buf, 6)
+		    || !strncasecmp("To: ", buf, 4)
+		    || !strncasecmp("Reply-", buf, 6)
+		    || !strncasecmp("Cc: ", buf, 4)
+		    || !strncasecmp("Bcc: ", buf, 5))
 	    strcpy(longbuf, buf);	
 	else if (longbuf[0])
 	{
