@@ -4,8 +4,7 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include <config.h>
-
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -34,7 +33,11 @@
 #endif
 #endif
 
-FILE *SockOpen(char *host, int clientPort)
+#ifdef SUNOS
+#include <memory.h>
+#endif
+
+int SockOpen(char *host, int clientPort)
 {
     int sock;
     unsigned long inaddr;
@@ -51,30 +54,30 @@ FILE *SockOpen(char *host, int clientPort)
     {
         hp = gethostbyname(host);
         if (hp == NULL)
-            return (FILE *)NULL;
+            return -1;
         memcpy(&ad.sin_addr, hp->h_addr, hp->h_length);
     }
     ad.sin_port = htons(clientPort);
     
     sock = socket(AF_INET, SOCK_STREAM, 0);
     if (sock < 0)
-        return (FILE *)NULL;
+        return -1;
     if (connect(sock, (struct sockaddr *) &ad, sizeof(ad)) < 0)
     {
 	close(sock);
-        return (FILE *)NULL;
+        return -1;
     }
 
-    return fdopen(sock, "r+");
+    return(sock);
 }
 
 
 #if defined(HAVE_STDARG_H)
-int SockPrintf(FILE *sockfp, char* format, ...)
+int SockPrintf(int sock, char* format, ...)
 {
 #else
-int SockPrintf(sockfp,format,va_alist)
-FILE *sockfp;
+int SockPrintf(sock,format,va_alist)
+int sock;
 char *format;
 va_dcl {
 #endif
@@ -87,20 +90,23 @@ va_dcl {
 #else
     va_start(ap);
 #endif
+#ifdef HAVE_VSNPRINTF
+    vsnprintf(buf, sizeof(buf), format, ap);
+#else
     vsprintf(buf, format, ap);
+#endif
     va_end(ap);
-    return SockWrite(buf, 1, strlen(buf), sockfp);
+    return SockWrite(sock, buf, strlen(buf));
 
 }
 
-int SockWrite(char *buf, int size, int len, FILE *sockfp)
+int SockWrite(int sock, char *buf, int len)
 {
     int n, wrlen = 0;
 
-    len *= size;
     while (len)
     {
-        n = write(fileno(sockfp), buf, len);
+        n = write(sock, buf, len);
         if (n <= 0)
             return -1;
         len -= n;
@@ -110,40 +116,41 @@ int SockWrite(char *buf, int size, int len, FILE *sockfp)
     return wrlen;
 }
 
-char *SockGets(char *buf, int len, FILE *sockfp)
+int SockRead(int sock, char *buf, int len)
 {
-    char *p, *bp = buf;
+    char *newline, *bp = buf;
     int n;
 
     if (--len < 1)
-	return NULL;
+	return(-1);
     do {
-	/* return value of 0 is EOF, < 0 is error */
-	if ((n = recv(fileno(sockfp), bp, len, MSG_PEEK)) <= 0)
-	    return NULL;
-	if ((p = memchr(bp, '\n', n)) != NULL)
-	{
-	    if (read(fileno(sockfp), bp, ++p - bp) == -1)
-		return NULL;
-	    *p = '\0';
-	    return buf;
-	}
-	if ((n = read(fileno(sockfp), bp, n)) == -1)
-	    return NULL;
+	/* 
+	 * The reason for these gymnastics is that we want two things:
+	 * (1) to read \n-terminated lines,
+	 * (2) to return the true length of data read, even if the
+	 *     data coming in has embedded NULS.
+	 */
+	if ((n = recv(sock, bp, len, MSG_PEEK)) <= 0)
+	    return(-1);
+	if ((newline = memchr(bp, '\n', n)) != NULL)
+	    n = newline - bp + 1;
+	if ((n = read(sock, bp, n)) == -1)
+	    return(-1);
 	bp += n;
 	len -= n;
-    } while (len);
+    } while 
+	    (!newline && len);
     *bp = '\0';
-    return buf;
+    return bp - buf;
 }
 
-int SockPeek(FILE *sockfp)
+int SockPeek(int sock)
 /* peek at the next socket character without actually reading it */
 {
     int n;
     char ch;
 
-    if ((n = recv(fileno(sockfp), &ch, 1, MSG_PEEK)) == -1)
+    if ((n = recv(sock, &ch, 1, MSG_PEEK)) == -1)
 	return -1;
     else
 	return(ch);
@@ -157,11 +164,11 @@ int SockPeek(FILE *sockfp)
  */
 main()
 {
-    FILE	*fp = SockOpen("localhost", 19);
+    int	 	sock = SockOpen("localhost", 19);
     char	buf[80];
 
-    while (SockGets(buf, sizeof(buf)-1, fp))
-	SockWrite(buf, 1, strlen(buf), stdout);
+    while (SockRead(sock, buf, sizeof(buf)-1))
+	SockWrite(1, buf, strlen(buf));
 }
 #endif /* MAIN */
 
