@@ -8,7 +8,7 @@
 #define SOCKET__
 
 /* Create a new client socket; returns (FILE *)NULL on error */
-FILE *SockOpen(char *host, int clientPort);
+int SockOpen(char *host, int clientPort);
 
 /* 
 Get a string terminated by an '\n' (matches interface of fgets).
@@ -16,25 +16,25 @@ Pass it a valid socket, a buffer for the string, and
 the length of the buffer (including the trailing \0)
 returns buffer on success, NULL on failure. 
 */
-char *SockGets(char *buf, int len, FILE *sockfp);
+int SockRead(int sock, char *buf, int len);
 
 /*
  * Peek at the next socket character without actually reading it.
  */
-int SockPeek(FILE *sockfp);
+int SockPeek(int sock);
 
 /*
 Write a chunk of bytes to the socket (matches interface of fwrite).
 Returns number of bytes successfully written.
 */
-int SockWrite(char *buf, int size, int nels, FILE *sockfp);
+int SockWrite(int sock, char *buf, int size);
 
 /* 
 Send formatted output to the socket (matches interface of fprintf).
 Returns number of bytes successfully written.
 */
 #if defined(HAVE_STDARG_H)
-int SockPrintf(FILE *sockfp, char *format, ...) ;
+int SockPrintf(int sock, char *format, ...) ;
 #else
 int SockPrintf();
 #endif
