@@ -1,11 +1,12 @@
 cat <<EOF
-Description: Remote mail fetcher for POP2, POP3, APOP, IMAP
+Description: Remote mail fetch daemon for POP2, POP3, APOP, IMAP
 Name: fetchmail
 Version: ${1}
-Release: ${2}
+Release: 1
 Source: locke.ccil.org:/pub/esr/fetchmail/fetchmail-${1}.tar.gz
-Copyright: distributable
+Copyright: freely redistributable
 Group: Mail
+Requires: smtpdaemon
 
 %prep
 %setup
@@ -18,7 +19,7 @@ make
 make install
 
 %files
-%doc README NEWS NOTES FAQ COPYING INSTALL sample.rcfile
+%doc README NEWS NOTES fetchmail-FAQ.html FAQ COPYING INSTALL sample.rcfile
 
 %ifarch i386
 %endif
