@@ -4,15 +4,13 @@
  * For license terms, see the file COPYING in this directory.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
-
 #if defined(STDC_HEADERS)
 #include <stdlib.h>
 #include <string.h>
 #endif
-
 #if defined(HAVE_UNISTD_H)
 #include <unistd.h>
 #endif
@@ -23,9 +21,7 @@
  * Machinery for handling UID lists live here.  This is mainly to support
  * RFC1725-conformant POP3 servers without a LAST command, but may also be
  * useful for making the IMAP4 querying logic UID-oriented, if a future
- * revision of IMAP forces me to.  (This would be bad.  Server-side 
- * seen bits are better than UIDs, because they track messages seen by
- * *all* clients.)
+ * revision of IMAP forces me to.
  *
  * Here's the theory:
  *
@@ -57,6 +53,8 @@
  * At the end of the fetchmail run, all current `oldsaved' lists are
  * flushed out to the .fetchids file to be picked up by the next run.
  * If there are no such messages, the file is deleted.
+ *
+ * Note: all comparisons are caseblind!
  */
 
 /* UIDs associated with un-queried hosts */
@@ -84,8 +82,9 @@ void initialize_saved_lists(struct query *hostlist, const char *idfile)
 	    {
 		for (ctl = hostlist; ctl; ctl = ctl->next)
 		{
-		    if (strcmp(host, ctl->server.names->id) == 0
-				&& strcmp(user, ctl->remotename) == 0)
+		    if (ctl->server.truename &&
+		        strcasecmp(host, ctl->server.truename) == 0
+				&& strcasecmp(user, ctl->remotename) == 0)
 		    {
 			save_str(&ctl->oldsaved, -1, id);
 			break;
@@ -112,7 +111,7 @@ struct idlist *save_str(struct idlist **idl, int num, const char *str)
 
     *end = (struct idlist *)xmalloc(sizeof(struct idlist));
     (*end)->val.num = num;
-    (*end)->id = xstrdup(str);
+    (*end)->id = str ? xstrdup(str) : (char *)NULL;
     (*end)->next = NULL;
 
     return(*end);
@@ -140,7 +139,7 @@ void save_str_pair(struct idlist **idl, const char *str1, const char *str2)
 	continue;
 
     *end = (struct idlist *)xmalloc(sizeof(struct idlist));
-    (*end)->id = xstrdup(str1);
+    (*end)->id = str1 ? xstrdup(str1) : (char *)NULL;
     if (str2)
 	(*end)->val.id2 = xstrdup(str2);
     else
@@ -174,6 +173,38 @@ int str_in_list(struct idlist **idl, const char *str)
 	return(str_in_list(&(*idl)->next, str));
 }
 
+int str_nr_in_list( struct idlist **idl, const char *str )
+  /* return the position of str in idl */
+{
+    int nr;
+    struct idlist *walk;
+    if ( !str )
+        return -1;
+    for( walk = *idl, nr = 0; walk; nr ++, walk = walk->next )
+        if( strcasecmp( str, walk->id) == 0 )
+	    return nr;
+    return -1;
+}
+
+int count_list( struct idlist **idl )
+  /* count the number of elements in the list */
+{
+  if( !*idl )
+    return 0;
+  return 1 + count_list( &(*idl)->next );
+}
+
+char* str_from_nr_list( struct idlist **idl, int number )
+  /* return the number'th string in idl */
+{
+    if( !*idl  || number < 0)
+        return 0;
+    if( number == 0 )
+        return (*idl)->id;
+    return str_from_nr_list(&(*idl)->next, number-1);
+}
+
+    
 char *str_find(struct idlist **idl, int number)
 /* return the id of the given number in the given list. */
 {
@@ -186,11 +217,11 @@ char *str_find(struct idlist **idl, int number)
 }
 
 char *idpair_find(struct idlist **idl, const char *id)
-/* return the id of the given number in the given list. */
+/* return the id of the given id in the given list (caseblind comparison) */
 {
     if (*idl == (struct idlist *) 0)
 	return((char *) 0);
-    else if (strcmp(id, (*idl)->id) == 0)
+    else if (strcasecmp(id, (*idl)->id) == 0)
 	return((*idl)->val.id2 ? (*idl)->val.id2 : (*idl)->id);
     else
 	return(idpair_find(&(*idl)->next, id));
@@ -257,7 +288,7 @@ void write_saved_lists(struct query *hostlist, const char *idfile)
 	    for (ctl = hostlist; ctl; ctl = ctl->next) {
 		for (idp = ctl->oldsaved; idp; idp = idp->next)
 		    fprintf(tmpfp, "%s@%s %s\n", 
-			    ctl->remotename, ctl->server.names->id, idp->id);
+			    ctl->remotename, ctl->server.truename, idp->id);
 	    }
 	    for (idp = scratchlist; idp; idp = idp->next)
 		fputs(idp->id, tmpfp);
